import java.util.Random;
/**
 * Class Matrix provides several static operation
 * on 2-dimensional arrays of double. The methods of class
 * Matrix can operate correctly on 2 dimensional arrays which
 * are rectangular (all rows  are of equal length) and
 * non-empty (not null and minimum size double[1][1]).
 * 
 * @author (Antonios Symvonis) 
 * @version (18 April 2002)
 */
public class Matrix
{
    /**
     * Prints the elements of a 2d-array, each row at a different line.
     * 
     * @param  m    The array of which the ellements are printed. 
     *              Must be rectangular and non-empty.
     */   
    public static void print(double m[][])
    {
        int rows=m.length;
        int cols=m[0].length;
        
        for( int i=0; i<rows; i++){
            for (int j=0; j<cols; j++)
                System.out.print(m[i][j] +"  ");
            System.out.println();
        }
        System.out.println();
    }//print

    /**
     * Creates an exact copy of a 2d-array.
     * 
     * @param  m    The array to be copied. Must be rectangular and non-empty.
     * @return      An exact copy of the argument array.
     */
    public static double[][] clone(double m[][])
    {
        int rows=m.length;
        double result[][]=new double[rows][];
        
        for(int i=0; i<rows; i++){
            int cols=m[i].length;
            result[i] = new double[cols];
            for(int j=0; j<cols; j++)
                result[i][j]=m[i][j];
        }
        return result;
    }//clone


    /**
     * Fills a  matrix with radmon data in the range [0..1). The random
     * numbers have precision of 2 decimal digits. 
     *
     * @param  m        The  array to be filled. Must be rectangular and non-empty.
     */
    public static void fillRandomData(double m[][])
    {
        Random random= new Random(0);
        int rows=m.length;
        int cols=m[0].length;
        
        for (int i=0; i<rows; i++)
            for (int j=0; j<cols; j++)
                m[i][j]=(double) random.nextInt(100)/100;
    }//fillRandomData
    
    /**
     * Computes the sum of two matrices of equal dimensions.
     *
     * @param  m1        First matrix to be added
     * @param  m2        Second matrix to be added
     */
    public static double[][] add(double m1[][], double m2[][])
    {
        int rows=m1.length;
        int cols=m1[0].length;
        double m[][] = new double[rows][cols];
        
        for (int i=0; i<rows; i++)
            for (int j=0; j<cols; j++)
                m[i][j]=m1[i][j] + m2[i][j];
                
        return m;
   }//add

    /**
     * Computes the difference of two matrices of equal dimensions.
     *
     * @param  m1        Minuend matrix
     * @param  m2        Subtrahend matrix
     */
    public static double[][] subtract(double m1[][], double m2[][])
    {
        int rows=m1.length;
        int cols=m1[0].length;
        double m[][] = new double[rows][cols];
        
        for (int i=0; i<rows; i++)
            for (int j=0; j<cols; j++)
                m[i][j]=m1[i][j] - m2[i][j];
                
        return m;
   }//subtract

    /**
     * Calculates the product of a matrix with a real number
     *
     * @param  r         Real number
     * @param  m1        Matrix to be multiplied
     */
    public static double[][] multiply(double r, double m1[][])
    {
        int rows=m1.length;
        int cols=m1[0].length;
        double m[][] = new double[rows][cols];
        
        for (int i=0; i<rows; i++)
            for (int j=0; j<cols; j++)
                m[i][j]=r * m1[i][j];
                
        return m;
   }//multiply

    /**
     * Calculates the internal product of two matrices
     *
     * @param  m1         First matrix
     * @param  m2         Second matrix
     */
    public static double[][] multiply(double m1[][], double m2[][])
    {
        int rows=m1.length;
        int cols=m2[0].length;
        int common=m1[0].length; // Could be common=m2.length
        double m[][] = new double[rows][cols];
        
        for (int i=0; i<rows; i++)
            for (int j=0; j<cols; j++) {
                m[i][j]=0;
                for (int k=0; k < common; k++)
                    m[i][j] += m1[i][k] * m2[k][j];
            }
                
        return m;
   }//multiply

    /**
     * Find the maximum of each row within the passed matrix.
     * Note: The dimension of the return matrix is 1 x rows.
     *
     * @param  m         Matrix to search
     */
    public static double[][] maxRows(double m[][])
    {
        int rows=m.length;
        int cols=m[0].length;
        double max[][] = new double[rows][1];
        
        for (int i=0; i<rows; i++) {
            max[i][0]=m[i][0];
            for (int j=1; j<cols; j++)
                if (m[i][j] > max[i][0])
                    max[i][0] = m[i][j];
        }
                
        return max;
   }//maxRows

    /**
     * Find the minimum of each row within the passed matrix.
     * Note: The dimension of the return matrix is 1 x rows.
     *
     * @param  m         Matrix to search
     */
    public static double[][] minRows(double m[][])
    {
        int rows=m.length;
        int cols=m[0].length;
        double min[][] = new double[rows][1];
        
        for (int i=0; i<rows; i++) {
            min[i][0]=m[i][0];
            for (int j=1; j<cols; j++)
                if (m[i][j] < min[i][0])
                    min[i][0] = m[i][j];
        }
                
        return min;
   }//minRows

    /**
     * Find the maximum of each column within the passed matrix.
     * Note: The dimension of the return matrix is cols x 1.
     *
     * @param  m         Matrix to search
     */
    public static double[][] maxCols(double m[][])
    {
        int rows=m.length;
        int cols=m[0].length;
        double max[][] = new double[1][cols];
        
        for (int j=0; j<cols; j++) {
            max[0][j]=m[0][j];
            for (int i=1; i<rows; i++)
                if (m[i][j] > max[0][j])
                    max[0][j] = m[i][j];
        }
                
        return max;
   }//maxCols

    /**
     * Find the minimum of each column within the passed matrix.
     * Note: The dimension of the return matrix is cols x 1.
     *
     * @param  m         Matrix to search
     */
    public static double[][] minCols(double m[][])
    {
        int rows=m.length;
        int cols=m[0].length;
        double min[][] = new double[1][cols];
        
        for (int j=0; j<cols; j++) {
            min[0][j]=m[0][j];
            for (int i=1; i<rows; i++)
                if (m[i][j] > min[0][j])
                    min[0][j] = m[i][j];
        }
                
        return min;
   }//minCols

    /**
     * Calculates the sum of each row of the passed matrix
     * Note: The dimension of the return matrix is 1 x rows.
     *
     * @param  m         Matrix to search
     */
    public static double[][] sumRows(double m[][])
    {
        int rows=m.length;
        int cols=m[0].length;
        double sum[][] = new double[rows][1];
        
        for (int i=0; i<rows; i++) {
            sum[i][0]=m[i][0];
            for (int j=1; j<cols; j++)
                sum[i][0] += m[i][j];
        }
                
        return sum;
   }//sumRows

    /**
     * Calculates the sum of each column of the passed matrix
     * Note: The dimension of the return matrix is cols x 1.
     *
     * @param  m         Matrix to search
     */
    public static double[][] sumCols(double m[][])
    {
        int rows=m.length;
        int cols=m[0].length;
        double sum[][] = new double[1][cols];
        
        for (int j=0; j<cols; j++) {
            sum[0][j]=m[0][j];
            for (int i=1; i<rows; i++)
                sum[0][j] += m[i][j];
        }
                
        return sum;
   }//sumCols

}//Matrix
